@@ -1,4 +1,4 @@
-#$Id: Agent.pm 347 2014-02-12 02:47:40Z maj $
+#$Id: Agent.pm 354 2014-02-17 05:22:50Z maj $
 use v5.10;
 package REST::Neo4p::Agent;
 use base LWP::UserAgent;
@@ -11,7 +11,7 @@ use warnings;
 
 our $VERSION;
 BEGIN {
-  $REST::Neo4p::Agent::VERSION = '0.2240';
+  $REST::Neo4p::Agent::VERSION = '0.2241';
 }
 
 our $AUTOLOAD;
@@ -251,14 +251,17 @@ sub __do_request {
       }
       $content = $JSON->encode($content) if $content;
       $resp  = $self->$rq($url, 'Content-Type' => 'application/json', Content=> $content, %$addl_headers);
+#      $DB::single=1;
+      1;
     }
   }
   # exception handling
   # rt80471...
-  eval {
-    $self->{_decoded_content} = $JSON->decode($resp->content);
-  };
-  undef $self->{_decoded_content} if $@;
+  if (length $resp->content) {
+    if ($resp->header('Content_Type') =~ /json/) {
+      $self->{_decoded_content} = $JSON->decode($resp->content);
+    }
+  }
   unless ($resp->is_success) {
     if ( $self->{_decoded_content} ) {
       my %error_fields = (
